package org.apache.bcel.verifier.statics;

import org.apache.bcel.verifier.exc.LocalVariableInfoInconsistentException;
import org.apache.bcel.generic.Type;
import java.util.Hashtable;

public class LocalVariableInfo
{
    private final Hashtable<String, Type> types;
    private final Hashtable<String, String> names;
    
    public LocalVariableInfo() {
        this.types = new Hashtable<String, Type>();
        this.names = new Hashtable<String, String>();
    }
    
    private void add(final int offset, final String name, final Type t) throws LocalVariableInfoInconsistentException {
        if (this.getName(offset) != null && !this.getName(offset).equals(name)) {
            throw new LocalVariableInfoInconsistentException("At bytecode offset '" + offset + "' a local variable has two different names: '" + this.getName(offset) + "' and '" + name + "'.");
        }
        if (this.getType(offset) != null && !this.getType(offset).equals(t)) {
            throw new LocalVariableInfoInconsistentException("At bytecode offset '" + offset + "' a local variable has two different types: '" + this.getType(offset) + "' and '" + t + "'.");
        }
        this.setName(offset, name);
        this.setType(offset, t);
    }
    
    public void add(final String name, final int startPc, final int length, final Type type) throws LocalVariableInfoInconsistentException {
        for (int i = startPc; i <= startPc + length; ++i) {
            this.add(i, name, type);
        }
    }
    
    public String getName(final int offset) {
        return this.names.get(Integer.toString(offset));
    }
    
    public Type getType(final int offset) {
        return this.types.get(Integer.toString(offset));
    }
    
    private void setName(final int offset, final String name) {
        this.names.put(Integer.toString(offset), name);
    }
    
    private void setType(final int offset, final Type t) {
        this.types.put(Integer.toString(offset), t);
    }
}
